'use strict';

exports.__esModule = true;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _events = require('events');

var _events2 = _interopRequireDefault(_events);

var _clusterize = require('clusterize.js');

var _clusterize2 = _interopRequireDefault(_clusterize);

var _flattree = require('flattree');

var _lookupTable = require('./lookup-table');

var _lookupTable2 = _interopRequireDefault(_lookupTable);

var _renderer = require('./renderer');

var _helper = require('./helper');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ensureNodeInstance = function ensureNodeInstance(node) {
    if (!(node instanceof _flattree.Node)) {
        throw new Error('The node must be a Node object.');
    }
    return true;
};

var extend = function extend(target) {
    for (var _len = arguments.length, sources = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
        sources[_key - 1] = arguments[_key];
    }

    if (target === undefined || target === null) {
        throw new TypeError('Cannot convert undefined or null to object');
    }

    var output = Object(target);
    for (var index = 0; index < sources.length; index++) {
        var source = sources[index];
        if (source !== undefined && source !== null) {
            for (var key in source) {
                if (source.hasOwnProperty(key)) {
                    output[key] = source[key];
                }
            }
        }
    }
    return output;
};

var InfiniteTree = function (_events$EventEmitter) {
    _inherits(InfiniteTree, _events$EventEmitter);

    // Creates new InfiniteTree object.

    function InfiniteTree() {
        var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

        _classCallCheck(this, InfiniteTree);

        // Assign options

        var _this = _possibleConstructorReturn(this, _events$EventEmitter.call(this));

        _this.options = {
            autoOpen: false,
            droppable: false,
            el: null,
            loadNodes: null,
            rowRenderer: _renderer.defaultRowRenderer,
            selectable: true,
            shouldSelectNode: null
        };
        _this.state = {
            openNodes: [],
            rootNode: null,
            selectedNode: null
        };
        _this.clusterize = null;
        _this.nodeTable = new _lookupTable2['default']();
        _this.nodes = [];
        _this.rows = [];
        _this.scrollElement = null;
        _this.contentElement = null;
        _this.dragoverElement = null;
        _this.contentListener = {
            'click': function click(e) {
                var itemTarget = null;
                var handleToggler = false;

                (0, _helper.stopPropagation)(e);

                if (e.target && e.currentTarget) {
                    itemTarget = e.target !== e.currentTarget ? e.target : null;
                } else if (e.srcElement) {
                    // IE8
                    itemTarget = e.srcElement;
                }

                while (itemTarget && itemTarget.parentElement !== _this.contentElement) {
                    if (itemTarget.className.indexOf('tree-toggler') >= 0) {
                        handleToggler = true;
                    }
                    itemTarget = itemTarget.parentElement;
                }

                if (!itemTarget) {
                    return;
                }

                var id = itemTarget.getAttribute('aria-id');
                var node = _this.getNodeById(id);

                if (!node) {
                    return;
                }

                // Click on the toggler to open/close a tree node
                if (handleToggler) {
                    _this.toggleNode(node);
                    return;
                }

                _this.selectNode(node);
            },
            // https://developer.mozilla.org/en-US/docs/Web/Events/dragenter
            // The dragenter event is fired when a dragged element or text selection enters a valid drop target.
            'dragenter': function dragenter(e) {
                var itemTarget = null;

                if (e.target && e.currentTarget) {
                    itemTarget = e.target !== e.currentTarget ? e.target : null;
                } else if (e.srcElement) {
                    // IE8
                    itemTarget = e.srcElement;
                }

                while (itemTarget && itemTarget.parentElement !== _this.contentElement) {
                    itemTarget = itemTarget.parentElement;
                }

                if (!itemTarget) {
                    return;
                }

                if (_this.dragoverElement !== itemTarget) {
                    (0, _helper.removeClass)(_this.dragoverElement, 'highlight'); // remove 'highlight' class
                    _this.dragoverElement = null;

                    if (!itemTarget.hasAttribute('droppable')) {
                        return;
                    }

                    var canDrop = !itemTarget.getAttribute('droppable').match(/false/i);
                    if (canDrop) {
                        (0, _helper.addClass)(itemTarget, 'highlight');
                        _this.dragoverElement = itemTarget;
                    }
                }
            },
            // https://developer.mozilla.org/en-US/docs/Web/Events/dragend
            // The dragend event is fired when a drag operation is being ended (by releasing a mouse button or hitting the escape key).
            'dragend': function dragend(e) {
                if (_this.dragoverElement) {
                    (0, _helper.removeClass)(_this.dragoverElement, 'highlight'); // remove 'highlight' class
                    _this.dragoverElement = null;
                }
            },
            // https://developer.mozilla.org/en-US/docs/Web/Events/dragover
            // The dragover event is fired when an element or text selection is being dragged over a valid drop target (every few hundred milliseconds).
            'dragover': function dragover(e) {
                (0, _helper.preventDefault)(e);
                e.dataTransfer.dropEffect = 'move';
                return false;
            },
            // https://developer.mozilla.org/en-US/docs/Web/Events/drop
            // The drop event is fired when an element or text selection is dropped on a valid drop target.
            'drop': function drop(e) {
                // prevent default action (open as link for some elements)
                (0, _helper.preventDefault)(e);

                if (_this.dragoverElement) {
                    var id = _this.dragoverElement.getAttribute('aria-id');
                    var node = _this.getNodeById(id);

                    (0, _helper.removeClass)(_this.dragoverElement, 'highlight');
                    _this.dragoverElement = null;

                    _this.emit('dropNode', node, e);
                }
            }
        };
        _this.options = extend({}, _this.options, options);

        if (!_this.options.el) {
            console.error('Failed to initialize infinite-tree: el is not specified.', options);
            return _possibleConstructorReturn(_this);
        }

        _this.create();

        // Load tree data if it's provided
        if (options.data) {
            _this.loadData(options.data);
        }
        return _this;
    }

    InfiniteTree.prototype.create = function create() {
        var _this2 = this;

        if (!this.options.el) {
            throw new Error('The element option is not specified.');
        }

        var scrollElement = document.createElement('div');
        scrollElement.className = (0, _helper.classNames)('infinite-tree', 'infinite-tree-scroll');
        var contentElement = document.createElement('div');
        contentElement.className = (0, _helper.classNames)('infinite-tree', 'infinite-tree-content');

        scrollElement.appendChild(contentElement);
        this.options.el.appendChild(scrollElement);

        this.clusterize = new _clusterize2['default']({
            tag: 'div',
            rows: [],
            scrollElem: scrollElement,
            contentElem: contentElement,
            no_data_class: 'infinite-tree-no-data',
            callbacks: {
                // Will be called right before replacing previous cluster with new one.
                clusterWillChange: function clusterWillChange() {},
                // Will be called right after replacing previous cluster with new one.
                clusterChanged: function clusterChanged() {},
                // Will be called on scrolling. Returns progress position.
                scrollingProgress: function scrollingProgress(progress) {
                    _this2.emit('scrollProgress', progress);
                }
            }
        });

        this.scrollElement = scrollElement;
        this.contentElement = contentElement;

        (0, _helper.addEventListener)(this.contentElement, 'click', this.contentListener.click);
        if (this.options.droppable) {
            (0, _helper.addEventListener)(document, 'dragend', this.contentListener.dragend);
            (0, _helper.addEventListener)(this.contentElement, 'dragenter', this.contentListener.dragenter);
            (0, _helper.addEventListener)(this.contentElement, 'dragover', this.contentListener.dragover);
            (0, _helper.addEventListener)(this.contentElement, 'drop', this.contentListener.drop);
        }
    };

    InfiniteTree.prototype.destroy = function destroy() {
        (0, _helper.removeEventListener)(this.contentElement, 'click', this.contentListener);
        if (this.options.droppable) {
            (0, _helper.removeEventListener)(document, 'dragend', this.contentListener.dragend);
            (0, _helper.removeEventListener)(this.contentElement, 'dragenter', this.contentListener.dragenter);
            (0, _helper.removeEventListener)(this.contentElement, 'dragover', this.contentListener.dragover);
            (0, _helper.removeEventListener)(this.contentElement, 'drop', this.contentListener.drop);
        }

        this.clear();

        if (this.clusterize) {
            this.clusterize.destroy(true); // True to remove all data from the list
            this.clusterize = null;
        }

        // Remove all child nodes
        while (this.contentElement.firstChild) {
            this.contentElement.removeChild(this.contentElement.firstChild);
        }
        while (this.scrollElement.firstChild) {
            this.scrollElement.removeChild(this.scrollElement.firstChild);
        }
        if (this.options.el) {
            var containerElement = this.options.el;
            while (containerElement.firstChild) {
                containerElement.removeChild(containerElement.firstChild);
            }
        }
        this.contentElement = null;
        this.scrollElement = null;
    };
    // Inserts a new child node to a parent node at the specified index.
    // * If the parent is null or undefined, inserts the child at the specified index in the top-level.
    // * If the parent has children, the method adds the child to it at the specified index.
    // * If the parent does not have children, the method adds the child to the parent.
    // * If the index value is greater than or equal to the number of children in the parent, the method adds the child at the end of the children.
    // @param {Object} newNode The new child node.
    // @param {number} index The 0-based index of where to insert the child node. Defaults to 0 for negative index.
    // @param {Node} parentNode The Node object that defines the parent node.
    // @return {boolean} Returns true on success, false otherwise.


    InfiniteTree.prototype.addChildNodeAt = function addChildNodeAt(newNode, index, parentNode) {
        var _this3 = this;

        // Defaults to rootNode if the parentNode is not specified
        parentNode = parentNode || this.state.rootNode;

        ensureNodeInstance(parentNode);

        if (!newNode) {
            return false;
        }
        index = Number(index) || 0;
        if (index < 0) {
            index = 0;
        }

        // Inserts the new child at the specified index
        newNode.parent = parentNode;
        parentNode.children.splice(index, 0, newNode);

        var deleteCount = parentNode.state.total;

        // Update index
        index = parentNode.children.indexOf(newNode);

        var nodes = (0, _flattree.flatten)(parentNode.children, { openNodes: this.state.openNodes });

        // Update newNode
        newNode = parentNode.getChildAt(index);

        var rows = nodes.map(function (node) {
            return _this3.options.rowRenderer(node, _this3.options);
        });
        var parentOffset = this.nodes.indexOf(parentNode);

        // Update nodes & rows
        this.nodes.splice.apply(this.nodes, [parentOffset + 1, deleteCount].concat(nodes));
        this.rows.splice.apply(this.rows, [parentOffset + 1, deleteCount].concat(rows));

        // Update the lookup table with newly added nodes
        this.flattenNode(newNode).forEach(function (node) {
            if (node.id !== undefined) {
                _this3.nodeTable.set(node.id, node);
            }
        });

        // Update the row corresponding to the parent node
        this.rows[parentOffset] = this.options.rowRenderer(parentNode, this.options);

        // Updates list with new data
        this.update();

        return true;
    };
    // Adds a node to the end of the list of children of a specified parent node.
    // * If the parent is null or undefined, inserts the child at the specified index in the top-level.
    // * If the parent has children, the method adds the child as the last child.
    // * If the parent does not have children, the method adds the child to the parent.
    // @param {Object} newNode The new child node.
    // @param {Node} parentNode The Node object that defines the parent node.
    // @return {boolean} Returns true on success, false otherwise.


    InfiniteTree.prototype.appendChildNode = function appendChildNode(newNode, parentNode) {
        // Defaults to rootNode if the parentNode is not specified
        parentNode = parentNode || this.state.rootNode;

        ensureNodeInstance(parentNode);

        var index = parentNode.children.length;
        return this.addChildNodeAt(newNode, index, parentNode);
    };
    // Clears the tree.


    InfiniteTree.prototype.clear = function clear() {
        this.clusterize.clear();
        this.nodeTable.clear();
        this.nodes = [];
        this.rows = [];
        this.state.openNodes = [];
        this.state.rootNode = null;
        this.state.selectedNode = null;
    };
    // Closes a node to hide its children.
    // @param {Node} node The Node object.
    // @return {boolean} Returns true on success, false otherwise.


    InfiniteTree.prototype.closeNode = function closeNode(node) {
        ensureNodeInstance(node);

        // Retrieve node index
        var nodeIndex = this.nodes.indexOf(node);
        if (nodeIndex < 0) {
            throw new Error('Invalid node index');
        }

        // Check if the closeNode action can be performed
        if (this.state.openNodes.indexOf(node) < 0) {
            return false;
        }

        // Keep selected node unchanged if "node" is equal to "this.state.selectedNode"
        if (this.state.selectedNode && this.state.selectedNode !== node) {
            // row #0 - node.0         => parent node (total=4)
            // row #1   - node.0.0     => close this node; next selected node (total=2)
            // row #2       node.0.0.0 => selected node (total=0)
            // row #3       node.0.0.1
            // row #4     node.0.1
            var selectedIndex = this.nodes.indexOf(this.state.selectedNode);
            var rangeFrom = nodeIndex + 1;
            var rangeTo = nodeIndex + node.state.total;

            if (rangeFrom <= selectedIndex && selectedIndex <= rangeTo) {
                this.selectNode(node);
            }
        }

        node.state.open = false; // Set the open state to false
        var openNodes = this.state.openNodes.filter(function (node) {
            return node.hasChildren() && node.state.open;
        });
        this.state.openNodes = openNodes;

        var deleteCount = node.state.total;

        // Subtract the deleteCount for all ancestors (parent, grandparent, etc.) of the current node
        for (var p = node; p !== null; p = p.parent) {
            p.state.total = p.state.total - deleteCount;
        }

        // Update nodes & rows
        this.nodes.splice(nodeIndex + 1, deleteCount);
        this.rows.splice(nodeIndex + 1, deleteCount);

        // Update the row corresponding to the node
        this.rows[nodeIndex] = this.options.rowRenderer(node, this.options);

        // Emit the 'closeNode' event
        this.emit('closeNode', node);

        // Updates list with new data
        this.update();

        return true;
    };
    // Flattens all child nodes of a parent node by performing full tree traversal using child-parent link.
    // No recursion or stack is involved.
    // @param {Node} parentNode The Node object that defines the parent node.
    // @return {array} Returns an array of Node objects containing all the child nodes of the parent node.


    InfiniteTree.prototype.flattenChildNodes = function flattenChildNodes(parentNode) {
        // Defaults to rootNode if the parentNode is not specified
        parentNode = parentNode || this.state.rootNode;

        ensureNodeInstance(parentNode);

        var list = [];

        // Ignore parent node
        var node = parentNode.getFirstChild();
        while (node) {
            list.push(node);
            if (node.hasChildren()) {
                node = node.getFirstChild();
            } else {
                // find the parent level
                while (node.getNextSibling() === null && node.parent !== parentNode) {
                    // use child-parent link to get to the parent level
                    node = node.getParent();
                }

                // Get next sibling
                node = node.getNextSibling();
            }
        }

        return list;
    };
    // Flattens a node by performing full tree traversal using child-parent link.
    // No recursion or stack is involved.
    // @param {Node} node The Node object.
    // @return {array} Returns a flattened list of Node objects.


    InfiniteTree.prototype.flattenNode = function flattenNode(node) {
        return [node].concat(this.flattenChildNodes(node));
    };
    // Gets a list of child nodes.
    // @param {Node} [parentNode] The Node object that defines the parent node. If null or undefined, returns a list of top level nodes.
    // @return {array} Returns an array of Node objects containing all the child nodes of the parent node.


    InfiniteTree.prototype.getChildNodes = function getChildNodes(parentNode) {
        // Defaults to rootNode if the parentNode is not specified
        parentNode = parentNode || this.state.rootNode;

        ensureNodeInstance(parentNode);

        return parentNode.children;
    };
    // Gets a node by its unique id. This assumes that you have given the nodes in the data a unique id.
    // @param {string|number} id An unique node id. A null value will be returned if the id doesn't match.
    // @return {Node} Returns a node the matches the id, null otherwise.


    InfiniteTree.prototype.getNodeById = function getNodeById(id) {
        var node = this.nodeTable.get(id);
        if (!node) {
            // Find the first node that matches the id
            node = this.nodes.filter(function (node) {
                return node.id === id;
            })[0];
            if (!node) {
                return null;
            }
            this.nodeTable.set(node.id, node);
        }
        return node;
    };
    // Gets an array of open nodes.
    // @return {array} Returns an array of Node objects containing open nodes.


    InfiniteTree.prototype.getOpenNodes = function getOpenNodes() {
        // returns a shallow copy of an array into a new array object.
        return this.state.openNodes.slice();
    };
    // Gets the root node.
    // @return {Node} Returns the root node, or null if empty.


    InfiniteTree.prototype.getRootNode = function getRootNode() {
        return this.state.rootNode;
    };
    // Gets the selected node.
    // @return {Node} Returns the selected node, or null if not selected.


    InfiniteTree.prototype.getSelectedNode = function getSelectedNode() {
        return this.state.selectedNode;
    };
    // Inserts the specified node after the reference node.
    // @param {Object} newNode The new sibling node.
    // @param {Node} referenceNode The Node object that defines the reference node.
    // @return {boolean} Returns true on success, false otherwise.


    InfiniteTree.prototype.insertNodeAfter = function insertNodeAfter(newNode, referenceNode) {
        ensureNodeInstance(referenceNode);
        var parentNode = referenceNode.getParent();
        var index = parentNode.children.indexOf(referenceNode) + 1;
        return this.addChildNodeAt(newNode, index, parentNode);
    };
    // Inserts the specified node before the reference node.
    // @param {Object} newNode The new sibling node.
    // @param {Node} referenceNode The Node object that defines the reference node.
    // @return {boolean} Returns true on success, false otherwise.


    InfiniteTree.prototype.insertNodeBefore = function insertNodeBefore(newNode, referenceNode) {
        ensureNodeInstance(referenceNode);
        var parentNode = referenceNode.getParent();
        var index = parentNode.children.indexOf(referenceNode);
        return this.addChildNodeAt(newNode, index, parentNode);
    };
    // Loads data in the tree.
    // @param {object|array} data The data is an object or array of objects that defines the node.


    InfiniteTree.prototype.loadData = function loadData() {
        var _this4 = this;

        var data = arguments.length <= 0 || arguments[0] === undefined ? [] : arguments[0];

        this.nodes = (0, _flattree.flatten)(data, { openAllNodes: this.options.autoOpen });

        // Clear lookup table
        this.nodeTable.clear();

        this.state.openNodes = this.nodes.filter(function (node) {
            return node.hasChildren() && node.state.open;
        });
        this.state.rootNode = function () {
            var node = arguments.length <= 0 || arguments[0] === undefined ? null : arguments[0];

            // Finding the root node
            while (node && node.parent !== null) {
                node = node.parent;
            }
            return node;
        }(this.nodes[0]);
        this.state.selectedNode = null;

        // Update the lookup table with newly added nodes
        this.flattenChildNodes(this.state.rootNode).forEach(function (node) {
            if (node.id !== undefined) {
                _this4.nodeTable.set(node.id, node);
            }
        });

        // Update rows
        this.rows = this.nodes.map(function (node) {
            return _this4.options.rowRenderer(node, _this4.options);
        });

        // Updates list with new data
        this.update();
    };
    // Opens a node to display its children.
    // @param {Node} node The Node object.
    // @return {boolean} Returns true on success, false otherwise.


    InfiniteTree.prototype.openNode = function openNode(node) {
        var _this5 = this;

        ensureNodeInstance(node);

        // Retrieve node index
        var nodeIndex = this.nodes.indexOf(node);
        if (nodeIndex < 0) {
            throw new Error('Invalid node index');
        }

        // Check if the openNode action can be performed
        if (this.state.openNodes.indexOf(node) >= 0) {
            return false;
        }

        if (!node.hasChildren() && node.loadOnDemand) {
            if (typeof this.options.loadNodes !== 'function') {
                return false;
            }

            // Set loading state to true
            node.state.loading = true;
            this.rows[nodeIndex] = this.options.rowRenderer(node, this.options);

            // Updates list with new data
            this.update();

            this.options.loadNodes(node, function (err, nodes) {
                // Set loading state to false
                node.state.loading = false;
                _this5.rows[nodeIndex] = _this5.options.rowRenderer(node, _this5.options);

                // Updates list with new data
                _this5.update();

                if (err) {
                    return;
                }

                // Append child nodes
                nodes.forEach(function (childNode) {
                    _this5.appendChildNode(childNode, node);
                });

                // Call openNode again
                _this5.openNode(node);
            });

            return false;
        }

        node.state.open = true; // Set node.state.open to true
        var openNodes = [node].concat(this.state.openNodes); // the most recently used items first
        this.state.openNodes = openNodes;

        var nodes = (0, _flattree.flatten)(node.children, { openNodes: this.state.openNodes });
        var rows = nodes.map(function (node) {
            return _this5.options.rowRenderer(node, _this5.options);
        });

        // Update nodes & rows
        this.nodes.splice.apply(this.nodes, [nodeIndex + 1, 0].concat(nodes));
        this.rows.splice.apply(this.rows, [nodeIndex + 1, 0].concat(rows));

        // Update the row corresponding to the node
        this.rows[nodeIndex] = this.options.rowRenderer(node, this.options);

        // Add all child nodes to the lookup table if the first child does not exist in the lookup table
        if (nodes.length > 0 && !this.nodeTable.get(nodes[0])) {
            nodes.forEach(function (node) {
                if (node.id !== undefined) {
                    _this5.nodeTable.set(node.id, node);
                }
            });
        }

        // Emit the 'openNode' event
        this.emit('openNode', node);

        // Updates list with new data
        this.update();

        return true;
    };
    // Removes all child nodes from a parent node.
    // @param {Node} parentNode The Node object that defines the parent node.
    // @return {boolean} Returns true on success, false otherwise.


    InfiniteTree.prototype.removeChildNodes = function removeChildNodes(parentNode) {
        var _this6 = this;

        ensureNodeInstance(parentNode);

        if (parentNode.children.length === 0) {
            return false;
        }

        var parentNodeIndex = this.nodes.indexOf(parentNode);

        // Update selected node
        if (parentNodeIndex >= 0 && this.state.selectedNode) {
            // row #0 - node.0         => parent node (total=4)
            // row #1   - node.0.0
            // row #2       node.0.0.0 => current selected node
            // row #3       node.0.0.1
            // row #4     node.0.1
            var selectedIndex = this.nodes.indexOf(this.state.selectedNode);
            var rangeFrom = parentNodeIndex + 1;
            var rangeTo = parentNodeIndex + parentNode.state.total;

            if (rangeFrom <= selectedIndex && selectedIndex <= rangeTo) {
                this.selectNode(parentNode);
            }
        }

        // Update parent node
        parentNode.children = [];
        parentNode.state.open = parentNode.state.open && parentNode.children.length > 0;

        // Get the number of nodes to be removed
        var deleteCount = parentNode.state.total;

        // Subtract the deleteCount for all ancestors (parent, grandparent, etc.) of the current node
        for (var p = parentNode; p !== null; p = p.parent) {
            p.state.total = p.state.total - deleteCount;
        }

        if (parentNodeIndex >= 0) {
            // Update nodes & rows
            this.nodes.splice(parentNodeIndex + 1, deleteCount);
            this.rows.splice(parentNodeIndex + 1, deleteCount);

            // Update the row corresponding to the parent node
            this.rows[parentNodeIndex] = this.options.rowRenderer(parentNode, this.options);
        }

        {
            (function () {
                // Update open nodes and lookup table
                var childNodes = _this6.flattenChildNodes(parentNode);

                _this6.state.openNodes = _this6.state.openNodes.filter(function (node) {
                    return childNodes.indexOf(node) < 0;
                });

                childNodes.forEach(function (node) {
                    _this6.nodeTable.unset(node.id);
                });
            })();
        }

        // Updates list with new data
        this.update();

        return true;
    };
    // Removes a node and all of its child nodes.
    // @param {Node} node The Node object.
    // @return {boolean} Returns true on success, false otherwise.


    InfiniteTree.prototype.removeNode = function removeNode(node) {
        var _this7 = this;

        ensureNodeInstance(node);

        var parentNode = node.parent;
        if (!parentNode) {
            return false;
        }

        // Retrieve node index
        var nodeIndex = this.nodes.indexOf(node);
        var parentNodeIndex = this.nodes.indexOf(parentNode);

        // Update selected node
        if (nodeIndex >= 0 && this.state.selectedNode) {
            // row #0 - node.0         => parent node (total=4)
            // row #1   - node.0.0     => remove this node (total=2)
            // row #2       node.0.0.0 => current selected node (total=0)
            // row #3       node.0.0.1
            // row #4     node.0.1     => next selected node (total=0)
            var selectedIndex = this.nodes.indexOf(this.state.selectedNode);
            var rangeFrom = nodeIndex;
            var rangeTo = nodeIndex + node.state.total + 1;

            if (rangeFrom <= selectedIndex && selectedIndex <= rangeTo) {
                // Change the selected node in the following order:
                // 1. next sibling node
                // 2. previous sibling node
                // 3. parent node
                var selectedNode = node.getNextSibling() || node.getPreviousSibling() || node.getParent();
                this.selectNode(selectedNode);
            }
        }

        // Update parent node
        parentNode.children.splice(parentNode.children.indexOf(node), 1);
        parentNode.state.open = parentNode.state.open && parentNode.children.length > 0;

        // Get the number of nodes to be removed
        var deleteCount = node.state.total + 1;

        // Subtract the deleteCount for all ancestors (parent, grandparent, etc.) of the current node
        for (var p = parentNode; p !== null; p = p.parent) {
            p.state.total = p.state.total - deleteCount;
        }

        if (nodeIndex >= 0) {
            // Update nodes & rows
            this.nodes.splice(nodeIndex, deleteCount);
            this.rows.splice(nodeIndex, deleteCount);
        }

        // Update the row corresponding to the parent node
        if (parentNodeIndex >= 0) {
            this.rows[parentNodeIndex] = this.options.rowRenderer(parentNode, this.options);
        }

        {
            (function () {
                // Update open nodes and lookup table
                var nodes = _this7.flattenNode(node);

                _this7.state.openNodes = _this7.state.openNodes.filter(function (node) {
                    return nodes.indexOf(node) < 0;
                });

                nodes.forEach(function (node) {
                    _this7.nodeTable.unset(node.id);
                });
            })();
        }

        // Updates list with new data
        this.update();

        return true;
    };
    // Sets the current scroll position to this node.
    // @param {Node} node The Node object.
    // @return {number} Returns the vertical scroll position, or -1 on error.


    InfiniteTree.prototype.scrollToNode = function scrollToNode(node) {
        ensureNodeInstance(node);

        // Retrieve node index
        var nodeIndex = this.nodes.indexOf(node);
        if (nodeIndex < 0) {
            return -1;
        }
        if (!this.contentElement) {
            return -1;
        }
        // Get the offset height of the first child element that contains the "tree-item" class
        var firstChild = this.contentElement.querySelectorAll('.tree-item')[0];
        var rowHeight = firstChild && firstChild.offsetHeight || 0;
        return this.scrollTop(nodeIndex * rowHeight);
    };
    // Gets (or sets) the current vertical position of the scroll bar.
    // @param {number} [value] An integer that indicates the new position to set the scroll bar to.
    // @return {number} Returns the vertical scroll position.


    InfiniteTree.prototype.scrollTop = function scrollTop(value) {
        if (!this.scrollElement) {
            return 0;
        }
        if (value !== undefined) {
            this.scrollElement.scrollTop = Number(value);
        }
        return this.scrollElement.scrollTop;
    };
    // Selects a node.
    // @param {Node} node The Node object. If null or undefined, deselects the current node.
    // @return {boolean} Returns true on success, false otherwise.


    InfiniteTree.prototype.selectNode = function selectNode() {
        var node = arguments.length <= 0 || arguments[0] === undefined ? null : arguments[0];
        var _options = this.options;
        var selectable = _options.selectable;
        var shouldSelectNode = _options.shouldSelectNode;


        if (!selectable) {
            return false;
        }
        if (typeof shouldSelectNode === 'function' && !shouldSelectNode(node)) {
            return false;
        }

        if (node === null) {
            // Deselect the current node
            if (this.state.selectedNode) {
                var selectedNode = this.state.selectedNode;
                var selectedIndex = this.nodes.indexOf(selectedNode);

                selectedNode.state.selected = false;
                this.rows[selectedIndex] = this.options.rowRenderer(selectedNode, this.options);
                this.state.selectedNode = null;

                // Emit the 'selectNode' event
                this.emit('selectNode', null);

                // Updates list with new data
                this.update();

                return true;
            }

            return false;
        }

        ensureNodeInstance(node);

        // Retrieve node index
        var nodeIndex = this.nodes.indexOf(node);
        if (nodeIndex < 0) {
            throw new Error('Invalid node index');
        }

        // Select this node
        if (this.state.selectedNode !== node) {
            node.state.selected = true;

            // Update the row corresponding to the node
            this.rows[nodeIndex] = this.options.rowRenderer(node, this.options);
        }

        // Deselect the current node
        if (this.state.selectedNode) {
            var _selectedNode = this.state.selectedNode;
            var _selectedIndex = this.nodes.indexOf(_selectedNode);
            _selectedNode.state.selected = false;
            this.rows[_selectedIndex] = this.options.rowRenderer(_selectedNode, this.options);
        }

        if (this.state.selectedNode !== node) {
            this.state.selectedNode = node;

            // Emit the 'selectNode' event
            this.emit('selectNode', node);
        } else {
            this.state.selectedNode = null;

            // Emit the 'selectNode' event
            this.emit('selectNode', null);
        }

        // Updates list with new data
        this.update();

        return true;
    };
    // Toggles a node to display or hide its children.
    // @param {Node} node The Node object.


    InfiniteTree.prototype.toggleNode = function toggleNode(node) {
        if (this.state.openNodes.indexOf(node) >= 0) {
            // close node
            this.closeNode(node);
        } else {
            // open node
            this.openNode(node);
        }
    };
    // Serializes the current state of a node to a JSON string.
    // @param {Node} node The Node object. If null, returns the whole tree.
    // @return {string} Returns a JSON string represented the tree.


    InfiniteTree.prototype.toString = function toString() {
        var node = arguments.length <= 0 || arguments[0] === undefined ? null : arguments[0];

        var traverse = function traverse(node) {
            var s = '[';
            if (node && node.children) {
                var _loop = function _loop(i) {
                    var list = [];
                    s = s + '{';
                    Object.keys(node).forEach(function (key) {
                        var value = node[key];
                        if (key === 'parent') {
                            // ignore parent
                            return;
                        }
                        if (key === 'children') {
                            // traverse child nodes
                            list.push('"' + key + '":' + traverse(node.children[i]));
                            return;
                        }
                        if (typeof value === 'string' || (typeof value === 'undefined' ? 'undefined' : _typeof(value)) === 'object') {
                            list.push('"' + key + '":' + JSON.stringify(value));
                        } else {
                            // primitive types
                            list.push('"' + key + '":' + value);
                        }
                    });
                    s = s + list.join(',');
                    s = s + '}' + (i === node.children.length - 1 ? '' : ',');
                };

                for (var i = 0; i < node.children.length; ++i) {
                    _loop(i);
                }
            }
            s = s + ']';
            return s;
        };

        if (!node) {
            node = this.state.rootNode;
        }

        return traverse(node);
    };
    // Updates the tree.


    InfiniteTree.prototype.update = function update() {
        // Update the list with new data
        this.clusterize.update(this.rows);

        // Emit the 'update' event
        this.emit('update');
    };
    // Updates the data of a node.
    // @param {Node} node The Node object.
    // @param {Object} data The data object.


    InfiniteTree.prototype.updateNode = function updateNode(node, data) {
        ensureNodeInstance(node);

        // The static attributes (i.e. children, parent, and state) are being protected
        var _node = node;
        var children = _node.children;
        var parent = _node.parent;
        var state = _node.state;

        node = extend(node, data, { children: children, parent: parent, state: state });

        // Retrieve node index
        var nodeIndex = this.nodes.indexOf(node);
        if (nodeIndex >= 0) {
            // Update the row corresponding to the node
            this.rows[nodeIndex] = this.options.rowRenderer(node, this.options);

            // Updates list with new data
            this.update();
        }
    };

    return InfiniteTree;
}(_events2['default'].EventEmitter);

exports['default'] = InfiniteTree;